package com.rjconsultores.ventaboletos.web.gui.controladores.catalogos;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.Categoria;
import com.rjconsultores.ventaboletos.entidad.MensagemRecusa;
import com.rjconsultores.ventaboletos.service.CategoriaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderMensagemRecusa;

@Controller("busquedaMensagemRecusaController")
@Scope("prototype")
public class BusquedaMensagemRecusaController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;

	@Autowired
	private CategoriaService categoriaService;

	@Autowired
	private transient PagedListWrapper<MensagemRecusa> plwMensagemRecusa;
	private MyListbox mensagemRecusaList;
	private Paging pagingMensagemRecusa;

	private Combobox cbmCategoria;
	private List<Categoria> lsCategoria;
	private Textbox txtDescricao;

	@Override
	public void doAfterCompose(Component comp) throws Exception {

		super.doAfterCompose(comp);
		mensagemRecusaList.setItemRenderer(new RenderMensagemRecusa());
		lsCategoria = categoriaService.obtenerTodos();

		mensagemRecusaList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				MensagemRecusa m = (MensagemRecusa) mensagemRecusaList.getSelected();
				verMensagemRecusa(m);
			}
		});

		refreshLista(false);
	}

	public void onClick$btnPesquisa(Event ev) {
		refreshLista(true);
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista(true);
	}

	public void onClick$btnNovo(Event ev) {
		verMensagemRecusa(new MensagemRecusa());
	}

	public void refreshLista(Boolean showmsg) {
		HibernateSearchObject<MensagemRecusa> sistemaBusqueda = new HibernateSearchObject<MensagemRecusa>(MensagemRecusa.class, pagingMensagemRecusa.getPageSize());

		String descricao = txtDescricao.getValue();
		if (descricao != null && StringUtils.isNotBlank(descricao)) {
			sistemaBusqueda.addFilterLike("descricao", "%".concat(descricao.trim().concat("%")));
		}

		Categoria categoria = cbmCategoria.getSelectedItem() != null ? (Categoria) cbmCategoria.getSelectedItem().getValue() : null;
		if (categoria != null) {
			sistemaBusqueda.addFilterEqual("categoria", categoria);
		}

		sistemaBusqueda.addFilterEqual("activo", Boolean.TRUE);

		plwMensagemRecusa.init(sistemaBusqueda, mensagemRecusaList, pagingMensagemRecusa);

		if (showmsg && mensagemRecusaList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaMensagemRecusa.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verMensagemRecusa(MensagemRecusa m) {
		if (m == null) {
			return;
		}

		Map args = new HashMap();
		args.put("mensagemRecusa", m);
		args.put("mensagemRecusaList", mensagemRecusaList);
		args.put("busquedaMensagemRecusaController", this);

		openWindow("/gui/catalogos/editarMensagemRecusa.zul",
				Labels.getLabel("editarMensagemRecusaController.window.title"), args, MODAL);
	}

	public MyListbox getMensagemRecusaList() {
		return mensagemRecusaList;
	}

	public void setMensagemRecusaList(MyListbox mensagemRecusaList) {
		this.mensagemRecusaList = mensagemRecusaList;
	}

	public Combobox getCbmCategoria() {
		return cbmCategoria;
	}

	public void setCbmCategoria(Combobox cbmCategoria) {
		this.cbmCategoria = cbmCategoria;
	}

	public Textbox getTxtDescricao() {
		return txtDescricao;
	}

	public void setTxtDescricao(Textbox txtDescricao) {
		this.txtDescricao = txtDescricao;
	}

	public List<Categoria> getLsCategoria() {
		return lsCategoria;
	}

	public void setLsCategoria(List<Categoria> lsCategoria) {
		this.lsCategoria = lsCategoria;
	}

}
